unit Main;
{***********************************************************************

  Copyright (C) 2004  Zaher Dirkey (zaher@parmaja.com)

  This file is part of Parmaja tools.

  PunTranslator is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunTranslator is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************}

interface

uses
  Forms, Controls, StdCtrls, ComCtrls, Classes, Windows, Messages, SysUtils,
  Registry, Contnrs, FileCtrl, Dialogs, Grids, MyParser, Menus;

type
  TMainForm = class(TForm)
    PageControl: TPageControl;
    TabSheet1: TTabSheet;
    KeyList: TListBox;
    VariableList: TListBox;
    ValueEdit: TMemo;
    Label2: TLabel;
    Label3: TLabel;
    KeyLbl: TEdit;
    EnValueEdit: TMemo;
    TabSheet2: TTabSheet;
    LogEdit: TMemo;
    Label5: TLabel;
    Label6: TLabel;
    MainMenu: TMainMenu;
    Files1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    Wizard1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Save1: TMenuItem;
    Label1: TLabel;
    Label10: TLabel;
    LclLangFolderEdit: TEdit;
    OrgLangFolderEdit: TEdit;
    Edit1: TMenuItem;
    Next1: TMenuItem;
    Find1: TMenuItem;
    FindinValues1: TMenuItem;
    FindNext1: TMenuItem;
    SaveAll1: TMenuItem;
    FindInLocalMnu: TMenuItem;
    FindinOriginal1: TMenuItem;
    N2: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure KeyListClick(Sender: TObject);
    procedure VariableListClick(Sender: TObject);
    procedure ValueEditChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure Exit1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Wizard1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Next1Click(Sender: TObject);
    procedure Find1Click(Sender: TObject);
    procedure FindinValues1Click(Sender: TObject);
    procedure FindNext1Click(Sender: TObject);
    procedure SaveAll1Click(Sender: TObject);
  private
    CurrentWord: TWordItem;
    Showed:Boolean;
    FindWordStr:string;
    procedure EnumVariables;
    procedure EnumValues;
    procedure ShowWizard;
    procedure LoadValue;
    procedure JumpTo(V,K:string);
    procedure FindFirst;
  public
    Files: TLangFiles;
    EnFiles: TLangFiles;
    procedure Enumerate;
    procedure LoadFiles(var Files: TLangFiles; Path: string);
    procedure SaveFiles(const Path:string; var Files: TLangFiles; Force:Boolean =False);
  end;

var
  MainForm: TMainForm;

implementation

uses WizardForms, AboutForms;

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  PageControl.ActivePageIndex := 0;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  aReg: TRegistry;
begin
  aReg := TRegistry.Create;
  aReg.OpenKey('Software\Options\PunTranslator', True);
  if OrgLangFolderEdit.Text<>'' then
    aReg.WriteString('OrginalLangPath', OrgLangFolderEdit.Text);
  if LclLangFolderEdit.Text<>'' then
    aReg.WriteString('LocalLangPath', LclLangFolderEdit.Text);
  aReg.Free;
  Files.Free;
  EnFiles.Free;
end;

procedure TMainForm.LoadFiles(var Files: TLangFiles; Path: string);
begin
  FreeAndNil(Files);
  Files := TLangFiles.Create;
  Files.Load(Path);
end;

procedure TMainForm.EnumVariables;
var
  i, j: Integer;
begin
  VariableList.Clear;
  for i := 0 to Files.Count - 1 do
  begin
    for j := 0 to Files.Items[i].Variables.Count - 1 do
    begin
      VariableList.Items.Add(Files.Items[i].Variables.Items[j].Name);
    end;
  end;
end;

procedure TMainForm.EnumValues;
var
  i: Integer;
  aVariable: TVariableItem;
begin
  KeyList.Clear;
  if VariableList.ItemIndex >= 0 then
  begin
    aVariable := Files[VariableList.Items[VariableList.ItemIndex]];
    for i := 0 to aVariable.Count - 1 do
    begin
      KeyList.Items.Add(aVariable.Items[i].Key);
    end;
  end;
end;

procedure TMainForm.KeyListClick(Sender: TObject);
begin
  LoadValue;
end;

procedure TMainForm.LoadValue;
var
  aCurrentWord: TWordItem;
begin
  if (KeyList.ItemIndex >= 0) and (VariableList.ItemIndex >= 0) then
  begin
    CurrentWord := nil; //locking ValueEdit.Changed
    aCurrentWord := Files[VariableList.Items[VariableList.ItemIndex]].Find(KeyList.Items[KeyList.ItemIndex]);
    ValueEdit.Text := aCurrentWord.Value;
    EnValueEdit.Text := EnFiles[VariableList.Items[VariableList.ItemIndex]][KeyList.Items[KeyList.ItemIndex]];
    KeyLbl.Text := VariableList.Items[VariableList.ItemIndex] + '\' + aCurrentWord.Key;
    CurrentWord := aCurrentWord;
  end
  else
  begin
    CurrentWord := nil;
    ValueEdit.Text := '';
    EnValueEdit.Text := '';
  end;
end;

procedure TMainForm.SaveFiles(const Path:string; var Files: TLangFiles; Force:Boolean);
begin
  Files.Save(Path, Force);
end;

procedure TMainForm.VariableListClick(Sender: TObject);
begin
  if VariableList.ItemIndex >= 0 then
  begin
    EnumValues;
  end;
end;

procedure TMainForm.ValueEditChange(Sender: TObject);
begin
  if CurrentWord <> nil then
    CurrentWord.Value := ValueEdit.Text;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  r: Integer;
begin
  if (Files <> nil) and Files.Changed then
  begin
    r := MessageBox(Handle, 'Files was changed, save before exit?', 'Save', MB_YESNOCANCEL);
    case r of
      MRYES: SaveFiles(LclLangFolderEdit.Text, Files);
      MRCANCEL: CanClose := False;
    end;
  end;
end;

procedure TMainForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.Save1Click(Sender: TObject);
begin
  SaveFiles(LclLangFolderEdit.Text, Files);
end;

procedure TMainForm.Wizard1Click(Sender: TObject);
begin
  ShowWizard;
end;

procedure TMainForm.Enumerate;
begin
  CurrentWord := nil;
  ValueEdit.Text := '';
  EnValueEdit.Text := '';
  if Files['lang_common']['lang_direction'] = 'rtl' then
    ValueEdit.BidiMode := bdRightToLeft
  else
    ValueEdit.BidiMode := bdLeftToRight;
  EnumVariables;
  VariableList.ItemIndex := 0;
  EnumVariables;
  KeyList.ItemIndex := 0;
  EnumValues;
end;

procedure TMainForm.ShowWizard;
begin
  with TWizardForm.Create(Application) do
  begin
    ShowModal;
    Free;
  end;
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
  if not Showed then
  begin
    Showed:=True;
    ShowWizard;
  end;
end;

procedure TMainForm.About1Click(Sender: TObject);
begin
  with TAboutForm.Create(Application) do
  begin
    ShowModal;
    Free;
  end;
end;

procedure TMainForm.Next1Click(Sender: TObject);
begin
  if KeyList.ItemIndex<KeyList.Items.Count-1 then
  begin
    KeyList.ItemIndex:=KeyList.ItemIndex+1;
    LoadValue;
  end
  else
  begin
    if VariableList.ItemIndex<VariableList.Items.Count-1 then
    begin
      VariableList.ItemIndex:=VariableList.ItemIndex + 1;
      EnumValues;
      KeyList.ItemIndex:=0;
      LoadValue;
    end
    else
      beep;
  end;
end;

procedure TMainForm.Find1Click(Sender: TObject);
var
  s:string;
  aWordItem:TWordItem;
  aFiles:TLangFiles;
begin
  if FindInLocalMnu.Checked then
    aFiles:=Files
  else
    aFiles:=EnFiles;
  if (aFiles<>nil) and InputQuery('Find','Enter Key name', s) then
  begin
    aWordItem:=aFiles.FindKey(s);
    if aWordItem<>nil then
    begin
      JumpTo(aWordItem.Variable.Name, aWordItem.Key);
    end;
  end;
end;

procedure TMainForm.JumpTo(V, K: string);
begin
  VariableList.ItemIndex:=VariableList.Items.IndexOf(V);
  if VariableList.ItemIndex>=0 then
  begin
    EnumValues;
    KeyList.ItemIndex:=KeyList.Items.IndexOf(K);
    LoadValue;
  end;
end;

procedure TMainForm.FindinValues1Click(Sender: TObject);
begin
  FindFirst;
end;

procedure TMainForm.FindFirst;
var
  aWordItem:TWordItem;
  aFiles:TLangFiles;  
begin
  if FindInLocalMnu.Checked then
    aFiles:=Files
  else
    aFiles:=EnFiles;
  if (Files<>nil) and InputQuery('Find Value','Enter any text to find', FindWordStr) then
  begin
    aWordItem:=aFiles.FindWordFirst(FindWordStr);
    if aWordItem<>nil then
    begin
      JumpTo(aWordItem.Variable.Name, aWordItem.Key);
    end
    else
      ShowMessage('Not found');
  end;
end;

procedure TMainForm.FindNext1Click(Sender: TObject);
var
  aWordItem:TWordItem;
  aFiles:TLangFiles;
begin
  if FindInLocalMnu.Checked then
    aFiles:=Files
  else
    aFiles:=EnFiles;
  if (Files<>nil) then
  begin
    if (FindWordStr='') then
      FindFirst
    else
    begin
      aWordItem:=aFiles.FindWordNext(FindWordStr);
      if aWordItem<>nil then
      begin
        JumpTo(aWordItem.Variable.Name, aWordItem.Key);
      end
      else
        ShowMessage('End of search');
    end;
  end;
end;

procedure TMainForm.SaveAll1Click(Sender: TObject);
begin
  SaveFiles(LclLangFolderEdit.Text, Files, True);
end;

end.

