unit MyParser;
{***********************************************************************

  Copyright (C) 2004  Zaher Dirkey (zaher@parmaja.com)

  This file is part of Parmaja tools.

  PunTranslator is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunTranslator is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Contnrs, RTLConsts, Dialogs;

const
  sOpenArray = 'array(';

type
  TRangeState = (rsUnKnown, rsArray);

  TVariableItem = class;
  TWordItem = class(TObject)
  private
    FChanged: Boolean;
    FVariable: TVariableItem;
    FKey: string;
    FValue: string;
    procedure SetValue(const Value: string);
  protected
    Line: Integer;
    Pos: Integer;
    Size: Integer;
    Checked: Boolean;
  public
    property Key: string read FKey write FKey;
    property Value: string read FValue write SetValue;
    property Variable: TVariableItem read FVariable;
    procedure Change;
  end;


  TLangFile = class;

  TWordList = class(TObjectList)
  private
    function GetItem(Index: Integer): TWordItem;
    procedure SetItem(Index: Integer; const Value: TWordItem);
  public
    property Items[Index: Integer]: TWordItem read GetItem write SetItem; default;
  end;

  TVariableItem = class(TWordList)
  private
    FChanged: Boolean;
    FLangFile: TLangFile;
    function GetValues(Index: string): string;
    procedure SetValues(Index: string; const Value: string);
  public
    Pos: Integer;
    New: Integer;
    Name: string;
    procedure Change;
    function Find(const vName: string): TWordItem;
    function FindWord(const Value: string): TWordItem;
    property Values[Index: string]: string read GetValues write SetValues; default;
  end;

  TVariableList = class(TObjectList)
  private
    function GetItem(Index: Integer): TVariableItem;
    procedure SetItem(Index: Integer; const Value: TVariableItem);
  public
    function Find(vName: string): TVariableItem;
    property Items[Index: Integer]: TVariableItem read GetItem write SetItem; default;
  end;

  TLangFiles = class;
  TLangFile = class(TObject)
  private
    FRange: TRangeState;
    FLine: Integer;
    FContains: TStringList;
    FFileName: string;
    FVariables: TVariableList;
    FChanged: Boolean;
    FLangFiles: TLangFiles;
  private
  public
    constructor Create;
    destructor Destroy; override;
    procedure Change;
    procedure ParseLine;
    procedure Parse;
    procedure ApplyChanges;
    procedure LoadFile(vFileName: string);
    procedure SaveFile(const Path:string; Force: Boolean);
    property Contains: TStringList read FContains;
    property Variables: TVariableList read FVariables;
    property FileName: string read FFileName;
  end;

  TLangFiles = class(TObjectList)
  private
    FChanged: Boolean;
    FIndex:TWordList;
    FLastIndex:Integer;
    function GetItem(Index: Integer): TLangFile;
    procedure SetItem(Index: Integer; const Value: TLangFile);
    function GetVariables(Index: string): TVariableItem;
    procedure CheckIndex;
    function FindWord(From:Integer; const S:string):TWordItem;
  public
    procedure Load(Path: string);
    procedure Change;
    procedure Save(const Path:string; Force:Boolean);
    property Changed: Boolean read FChanged;
    function Compare(vFiles: TLangFiles; Log: TStrings): Boolean;
    function Import(vFiles: TLangFiles; Log: TStrings): Boolean;
    function FindKey(S:string):TWordItem;
    function FindWordFirst(const S:string):TWordItem;
    function FindWordNext(const S:string):TWordItem;
    property Variables[Index: string]: TVariableItem read GetVariables; default;
    property Items[Index: Integer]: TLangFile read GetItem write SetItem;
  end;

implementation

uses StrUtils;

{ TLangFile }

procedure TLangFile.LoadFile(vFileName: string);
begin
  FFileName := ExtractFileName(vFileName);
  FContains := TStringList.Create;
  FContains.LoadFromFile(vFileName);
  Parse;
end;

procedure TLangFile.ParseLine;
var
  i: Integer;
  s, aLine: string;
  aOpened: Boolean;
//  aOpenChar: Char;//not impl yet for both " ' supporting
  aOpenPos, aOpenCount: Integer;
  aKey, aValue: string;
  aVariable: TVariableItem;
  aWord: TWordItem;
  CurChar, NextChar: Char;
begin
  aLine := FContains[FLine];
  s := Trim(aLine);
  case FRange of
    rsUnKnown:
      begin
        if (s <> '') and (s[1] = '$') and (RightStr(s, Length(sOpenArray)) = sOpenArray) then
        begin
          aVariable := TVariableItem.Create;
          aVariable.FLangFile := Self;
          aVariable.Name := Copy(s, 2, AnsiPos(' ', s) - 2);
          FVariables.Add(aVariable);
          FRange := rsArray;
        end;
      end;
    rsArray:
      begin
        aOpenPos := 0;
        aOpened := False;
        aOpenCount := 0;
        i := 1;
        while i <= Length(aLine) do
        begin
          CurChar := aLine[i];
          if i < Length(aLine) then
            NextChar := aLine[i + 1]
          else
            NextChar := #0;
          if not aOpened and (CurChar = ')') and (NextChar = ';') then
          begin
            FRange := rsUnKnown;
            Break;
          end
          else
            if not aOpened and (CurChar = '/') and (NextChar = '/') then
            begin
              Break;
            end
            else
              if aOpened and (CurChar = '\') then
              begin
                Inc(i); //skip char
              end
              else
                if CurChar = '''' then
                begin
                  if not aOpened then
                  begin
                    aOpenPos := i;
                    Inc(aOpenCount);
                    aOpened := True;
                  end
                  else
                  begin
                    if aOpenCount = 1 then
                      aKey := Copy(aLine, aOpenPos + 1, i - aOpenPos - 1)
                    else
                    begin
                      aValue := Copy(aLine, aOpenPos + 1, i - aOpenPos - 1);
                      aVariable := FVariables[FVariables.Count - 1];
                      with aVariable do
                      begin
                        aWord := TWordItem.Create;
                        aWord.Line := FLine;
                        aWord.Pos := aOpenPos + 1;
                        aWord.Size := i - aOpenPos - 1;
                        aWord.Key := aKey;
                        aWord.FValue := aValue;
                        aWord.FVariable := aVariable;
                        Add(aWord);
                      end;
                    end;
                    aOpened := False;
                  end
                end;
          Inc(i);
        end;
      end;
  end;
end;

procedure TLangFile.Parse;
begin
  FLine := 0;
  while FLine < FContains.Count do
  begin
    ParseLine;
    Inc(FLine);
  end;
end;

constructor TLangFile.Create;
begin
  inherited;
  FVariables := TVariableList.Create;
end;

destructor TLangFile.Destroy;
begin
  FVariables.Free;
  inherited;
end;

procedure TLangFile.SaveFile(const Path:string; Force: Boolean);
begin
  if FChanged or Force then
  begin
    ApplyChanges;
    FContains.SaveToFile(IncludeTrailingPathDelimiter(Path)+FileName);
    FChanged := False;
  end;
end;

procedure TLangFile.ApplyChanges;
var
  i, j: Integer;
  s: string;
begin
  for i := 0 to Variables.Count - 1 do
  begin
    for j := 0 to Variables.Items[i].Count - 1 do
    begin
      with Variables.Items[i].Items[j] do
        if FChanged then
        begin
          s := FContains[Line];
          Delete(s, Pos, Size);
          Insert(Value, s, Pos);
          Size := Length(FValue);
          FContains[Line] := s;
          FChanged := False;
        end;
      Variables.Items[i].FChanged := False;
    end;
  end;
end;

procedure TLangFile.Change;
begin
  FChanged := True;
  FLangFiles.Change;
end;

{ TLangFiles }

procedure TLangFiles.Change;
begin
  FChanged := True;
end;

procedure TLangFiles.CheckIndex;
var
  i, j, k: Integer;
begin
  if FIndex=nil then
  begin
    FIndex:=TWordList.Create;
    for i := 0 to Count - 1 do
    begin
      for j := 0 to Items[i].Variables.Count - 1 do
      begin
        for k := 0 to Items[i].Variables[j].Count - 1 do
        begin
          FIndex.Add(Items[i].Variables[j].Items[k]);
        end;
      end;
    end;
  end;
end;


function TLangFiles.Compare(vFiles: TLangFiles; Log: TStrings): Boolean;
var
  i, j, k: Integer;
  aChecked:Boolean;
begin
  Result := True;
  for i := 0 to Count - 1 do
  begin
    for j := 0 to Items[i].Variables.Count - 1 do
    begin
      for k := 0 to Items[i].Variables[j].Count - 1 do
      begin
        aChecked:=vFiles[Items[i].Variables[j].Name].Find(Items[i].Variables[j].Items[k].Key) <> nil;
        Items[i].Variables[j].Items[k].Checked := aChecked;
        if not aChecked then
        begin
          Log.Add(Items[i].FileName + ' Line:' + IntToStr(Items[i].Variables[j].Items[k].Line + 1) + ' : ' + Items[i].Variables[j].Name + '=>' + Items[i].Variables[j].Items[k].Key);
          Result := False;
        end;
      end;
    end;
  end;
end;

function TLangFiles.FindWord(From:Integer; const S: string): TWordItem;
var
  i:Integer;
begin
  CheckIndex;
  Result := nil;
  for i := From to FIndex.Count - 1 do
  begin
    if AnsiContainsText(FIndex[i].Value, S) then
    begin
      FLastIndex:=i;
      Result:=FIndex[i];
      exit;
    end;
  end;
end;

function TLangFiles.FindKey(S: string): TWordItem;
var
  i:Integer;
begin
  CheckIndex;
  Result := nil;
  for i := 0 to FIndex.Count - 1 do
  begin
    if SameText(FIndex[i].Key, S) then
    begin
      Result:=FIndex[i];
      exit;
    end;
  end;
end;

function TLangFiles.GetItem(Index: Integer): TLangFile;
begin
  Result := inherited Items[Index] as TLangFile;
end;

function TLangFiles.GetVariables(Index: string): TVariableItem;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
  begin
    Result := Items[i].Variables.Find(Index);
    if Result <> nil then
      break;
  end;
end;

function TLangFiles.Import(vFiles: TLangFiles; Log: TStrings): Boolean;
var
  i, j, k: Integer;
  aOrgWordItem:TWordItem;
  aWordItem:TWordItem;
begin
  Result := True;
  for i := 0 to Count - 1 do
  begin
    for j := 0 to Items[i].Variables.Count - 1 do
    begin
      for k := 0 to Items[i].Variables[j].Count - 1 do
      begin
        aOrgWordItem:=Items[i].Variables[j].Items[k];
        aWordItem:=vFiles[Items[i].Variables[j].Name].Find(aOrgWordItem.Key);
        if aWordItem <> nil then
        begin
          aOrgWordItem.Value:= aWordItem.Value;
          aOrgWordItem.Change;
          aOrgWordItem.Checked := True;
        end
        else
        begin
          Log.Add(Items[i].FileName + ' Line:' + IntToStr(aOrgWordItem.Line + 1) + ' : ' + Items[i].Variables[j].Name + '=>' + aOrgWordItem.Key);
          Result := False;
        end;
      end;
    end;
  end;
end;

procedure TLangFiles.Load(Path: string);
var
  I: Integer;
  SearchRec: TSearchRec;
  aLangFile: TLangFile;
begin
  Path := IncludeTrailingPathDelimiter(Path);
  try
    I := FindFirst(Path + '*.php', 0, SearchRec);
    while I = 0 do
    begin
      aLangFile := TLangFile.Create;
      aLangFile.LoadFile(Path + SearchRec.Name);
      aLangFile.FLangFiles := Self;
      Add(aLangFile);
      I := FindNext(SearchRec);
    end;
    FindClose(SearchRec);
  except
    raise;
  end;
end;

procedure TLangFiles.Save(const Path:string; Force:Boolean);
var
  i: Integer;
begin
  for i := 0 to Count - 1 do
  begin
    Items[i].SaveFile(Path, Force);
  end;
  FChanged := False;
end;

procedure TLangFiles.SetItem(Index: Integer; const Value: TLangFile);
begin
  inherited Items[Index] := Value;
end;

function TLangFiles.FindWordNext(const S: string): TWordItem;
begin
  Result:=FindWord(FLastIndex + 1, S);
end;

function TLangFiles.FindWordFirst(const S: string): TWordItem;
begin
  Result:=FindWord(0, S);
end;

{ TVariableItem }

procedure TVariableItem.Change;
begin
  FChanged := True;
  FLangFile.Change;
end;

function TVariableItem.Find(const vName: string): TWordItem;
var
  i: Integer;
begin
  Result := nil;
  if Self <> nil then
    for i := 0 to Count - 1 do
    begin
      if Items[i].Key = vName then
      begin
        Result := Items[i];
        Break;
      end;
    end;
end;

function TVariableItem.FindWord(const Value: string): TWordItem;
var
  i: Integer;
begin
  Result := nil;
  if Self <> nil then
    for i := 0 to Count - 1 do
    begin
      if AnsiContainsText(Items[i].Value, Value) then
      begin
        Result := Items[i];
        Break;
      end;
    end;
end;

function TWordList.GetItem(Index: Integer): TWordItem;
begin
  Result := inherited Items[Index] as TWordItem;
end;

function TVariableItem.GetValues(Index: string): string;
var
  aWordItem: TWordItem;
begin
  if Self <> nil then
  begin
    aWordItem := Find(Index);
    if aWordItem <> nil then
      Result := aWordItem.FValue
    else
      Result := '';
  end
  else
    Result := '';
end;

procedure TWordList.SetItem(Index: Integer; const Value: TWordItem);
begin
  inherited Items[Index] := Value;
end;

procedure TVariableItem.SetValues(Index: string; const Value: string);
var
  aWordItem: TWordItem;
begin
  aWordItem := Find(Index);
  if aWordItem <> nil then
    aWordItem.Value := Value;
end;

{ TVariableList }

function TVariableList.Find(vName: string): TVariableItem;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
  begin
    if Items[i].Name = vName then
    begin
      Result := Items[i];
      Break;
    end;
  end;
end;

function TVariableList.GetItem(Index: Integer): TVariableItem;
begin
  Result := inherited Items[Index] as TVariableItem;
end;

procedure TVariableList.SetItem(Index: Integer; const Value: TVariableItem);
begin
  inherited Items[Index] := Value;
end;

{ TWordItem }

procedure TWordItem.Change;
begin
  FChanged := True;
  FVariable.Change;
end;

procedure TWordItem.SetValue(const Value: string);
begin
  FValue := Value;
  Change;
end;

end.

